// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/12AI64/12AI64_Linux_1.x.x.x_DN/samples/fsamp/perform.c $
// $Rev: 55669 $
// $Date: 2024-10-23 12:19:03 -0500 (Wed, 23 Oct 2024) $

// 12AI64: Sample Application: source file

#include "main.h"



// macros *********************************************************************

#define	FILE_NAME				"fsamp.txt"
#define	LIMIT_MAX(v,max)		(((v) > (max)) ? (max) : (v))
#define	LIMIT_MIN(v,min)		(((v) < (min)) ? (min) : (v))
#define	LIMIT_RANGE(v,min,max)	(LIMIT_MIN((LIMIT_MAX((v),(max))),(min)))



//*****************************************************************************
static int _hold_secs(const args_t* args, s32 fsamp)
{
	char	buf[80];
	int		errs	= 0;

	gsc_label("Config");
	errs	+= ai64_config_ai(args->fd, -1, 0, AI64_AI_SCAN_SIZE_0_15, fsamp);
	printf("%s\n", errs ? "FAIL <---" : "PASS");

	if (args->hold_reg_dump)
		ai64_reg_list(args->fd, 1);

	strcpy(buf, "Holding ");
	gsc_label_long_comma_buf(args->hold_secs, buf + strlen(buf));
	strcat(buf, " secs");
	gsc_label(buf);

	if (errs == 0)
	{
		os_sleep_ms(args->hold_secs * 1000);
		printf("Done\n");
	}
	else
	{
		printf("Aborting due to error.\n");
	}

	return(errs);
}



//*****************************************************************************
static int _plus_minus(const args_t* args)
{
	char	buf[80];
	s32		chan_qty;
	int		count	= 0;
	int		errs	= 0;
	s32		fconv;
	double	fgen;
	s32		fgen_max;
	s32		fref;
	double	fsamp;
	s32		fsamp_max;
	s32		fsamp_min;
	double	limit_max;
	double	limit_min;
	s32		nrate_a;
	s32		nrate_b;
	s32		nrate_max;
	s32		nrate_min;
	s32		scan_size;

	errs	+= ai64_query(args->fd, -1, 1, AI64_QUERY_MASTER_CLOCK, &fref);
	errs	+= ai64_query(args->fd, -1, 0, AI64_QUERY_FSAMP_MAX, &fsamp_max);
	errs	+= ai64_query(args->fd, -1, 0, AI64_QUERY_FSAMP_MIN, &fsamp_min);
	errs	+= ai64_query(args->fd, -1, 1, AI64_QUERY_NRATE_MIN, &nrate_min);
	errs	+= ai64_query(args->fd, -1, 1, AI64_QUERY_NRATE_MAX, &nrate_max);

	errs	+= ai64_ai_scan_size(args->fd, -1, 1, -1, &scan_size);

	switch (scan_size)
	{
		default:						errs++;
		case AI64_AI_SCAN_SIZE_SINGLE:	chan_qty	= 1;	break;
		case AI64_AI_SCAN_SIZE_0_1:		chan_qty	= 2;	break;
		case AI64_AI_SCAN_SIZE_0_3:		chan_qty	= 4;	break;
		case AI64_AI_SCAN_SIZE_0_7:		chan_qty	= 8;	break;
		case AI64_AI_SCAN_SIZE_0_15:	chan_qty	= 16;	break;
		case AI64_AI_SCAN_SIZE_0_31:	chan_qty	= 32;	break;
		case AI64_AI_SCAN_SIZE_0_63:	chan_qty	= 64;	break;
	}

	fconv	= (chan_qty == 1) ? 1500000 : 1000000;
	fsamp	= LIMIT_RANGE(args->fsamp, fsamp_min, fsamp_max);

	gsc_label("Fsamp Computation");
	gsc_label_long_comma(fsamp);
	printf(" S/S, +- ");
	gsc_label_long_comma(args->plus_minus);
	printf(" S/S\n");
	gsc_label_level_inc();

	limit_min	= fsamp - args->plus_minus;
	limit_max	= fsamp + args->plus_minus;
	fgen_max	= fconv / chan_qty;

	for (nrate_b = nrate_min - 1; nrate_b <= nrate_max; nrate_b++)
	{
		for (nrate_a = nrate_min; nrate_a <= nrate_max; nrate_a++)
		{
			if (nrate_b >= nrate_min)
				fgen	= (double) fref / (nrate_a * nrate_b);
			else
				fgen	= (double) fref / nrate_a;

			if (fgen > fgen_max)
				continue;

			fsamp	= fgen / chan_qty;

			if (fsamp < limit_min)
			{
				//printf(" <<<\n");
				continue;
			}

			if (fsamp > limit_max)
			{
				//printf(" >>>\n");
				continue;
			}

			sprintf(buf, "%d ", count);
			gsc_label_float_comma_buf(fsamp, 0, 3, buf + strlen(buf));
			gsc_label(buf);

			printf("Fref ");
			gsc_label_long_comma(fref);

			printf(", Nrate A ");
			gsc_label_long_comma(nrate_a);

			if (nrate_b >= nrate_min)
			{
				printf(", Nrate b ");
				gsc_label_long_comma(nrate_b);
			}

			printf("\n");
			count++;
		}
	}

	gsc_label("Options");

	if (count == 0)
	{
		errs++;
		printf("FAIL <---");
	}
	else
	{
		printf("PASS");
	}

	printf("  (");
	gsc_label_long_comma(count);
	printf(")\n");

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
static int _range(const args_t* args)
{
	int			errs	= 0;
	FILE*		file	= NULL;
	s32			fsamp;
	s32			max		= args->fsamp;
	s32			min		= args->fsamp;
	const char*	name	= FILE_NAME;
	double		rate;

	errs	+= ai64_query(args->fd, -1, 0, AI64_QUERY_FSAMP_MAX, &max);
	errs	+= ai64_query(args->fd, -1, 0, AI64_QUERY_FSAMP_MIN, &min);

	gsc_label("Fsamp Computation");
	gsc_label_level_inc();

	if (args->range == 0)
	{
		min	= LIMIT_RANGE(args->fsamp, min, max);
		max	= min;

		gsc_label_long_comma(min);
		printf(" S/S\n");
	}
	else
	{
		if (args->range_begin)
			min	= LIMIT_RANGE(args->range_begin, min, max);

		if (args->range_end)
			max	= LIMIT_RANGE(args->range_end, min, max);

		gsc_label_long_comma(min);
		printf(" S/S to ");

		gsc_label_long_comma(max);
		printf(" S/S\n");
	}

	if (args->save)
	{
		file	= fopen(name, "w+b");

		if (file == NULL)
		{
			printf("FAIL <---  (unable to create %s)\n", name);
			errs	= 1;
		}
		else
		{
			fprintf(file, "# 12AI64 Sample Rates\n");
			fprintf(file, "# Request  Produced\n");
			fprintf(file, "# =======  ==========\n");
		}
	}

	for (fsamp = min; fsamp <= max; fsamp++)
	{
		errs	+= ai64_fsamp_ai_compute(args->fd, -1, 1, fsamp, NULL, NULL, NULL, NULL, NULL, NULL, &rate);

		if (args->hold_secs)
			errs	+= _hold_secs(args, fsamp);

		if (file)
			fprintf(file, "%9ld  %10.3f\n", (long) fsamp, (float) rate);
	}

	if (file)
		fclose(file);

	gsc_label_level_dec();
	return(errs);
}



//*****************************************************************************
int perform_tests(const args_t* args)
{
	int	errs;

	if (args->plus_minus >= 0)
		errs	= _plus_minus(args);
	else
		errs	= _range(args);

	return(errs);
}


