// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/12AI64/12AI64_Linux_1.x.x.x_DN/utils/irq1_sel.c $
// $Rev: 55668 $
// $Date: 2024-10-23 12:14:41 -0500 (Wed, 23 Oct 2024) $

// 12AI64: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ai64_irq1_sel
*
*	Purpose:
*
*		Provide a visual wrapper for the AI64_IOCTL_IRQ1_SEL service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ai64_irq1_sel(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	int			ret;

	if (verbose)
		gsc_label_index("IRQ1 Select", index);

	ret		= ai64_ioctl(fd, AI64_IOCTL_IRQ1_SEL, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case -1:

			ptr	= "This service is not supported by this device.";
			break;

		case AI64_IRQ1_NONE:

			ptr	= "None";
			break;

		case AI64_IRQ1_IN_BUF_THR_L2H:

			ptr	= "Input Buffer Threshold Low-to-High";
			break;

		case AI64_IRQ1_IN_BUF_THR_H2L:

			ptr	= "Input Buffer Threshold High-to-Low";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


